#define _WIN32_DCOM

#import "../Commun/Commun___Win32_Debug/premier.tlb"
#include "../Commun/Fabriques--ServeurPremiers.h"
#include <iostream>
#include <string>

#include <conio.h> // getch ()


// Petite procdure pour flusher COM et terminer de manire
// semi brutale le client...
void Terminer (const std::string &msg, const HRESULT hr)
{
   std::cerr << msg << ", code " << hr << std::endl
             << "Pressez une touche pour terminer...";
   getch ();
   CoUninitialize ();
   exit (-1);
}

// Petite procdure interactive pour identifier des entiers
// choisis par l'usager comme tant premiers ou non. Est nave
// pour ce qui est du caractre asynchrone
void IdentifierDesPremiers (COMAnalyserPremiers::AsyncIPremier *);

// Petite procdure interactive pour trouver des entiers premiers
// (un meilleur exemple de gestion asynchrone ct client)
void TrouverDesPremiers (COMAnalyserPremiers::AsyncIPremier *);

void main ()
{
   // Pas besoin de multithreading pour ce client
   HRESULT hr = CoInitializeEx (0, COINIT_APARTMENTTHREADED);
   if (FAILED (hr))
      Terminer ("Echec de CoInitializeEx ()", hr);

   // Cration du serveur
   IUnknown *pUnk;
   hr = CoCreateInstance ( CLSID_FabriqueAnalystePremiers,
                           0,
                           CLSCTX_INPROC_SERVER,
                           __uuidof(IUnknown),
                           (void **) &pUnk);
   if (FAILED (hr))
      Terminer ("Echec de CoCreateInstance ()", hr);

   ICallFactory *pAppel;
   // Obtention de la fabrique d'objets d'appel
   hr = pUnk->QueryInterface (__uuidof(ICallFactory), (void **) &pAppel);
   pUnk->Release ();
   if (FAILED(hr))
      Terminer ("Echec  l'obtention de ICallFactory", hr);

   // Obtention de l'objet d'appel
   IUnknown *pAsyncUnk;
   hr = pAppel->CreateCall (__uuidof (COMAnalyserPremiers::AsyncIPremier),
                            0,
                            IID_IUnknown,
                            &pAsyncUnk);
   pAppel->Release ();
   if (FAILED (hr))
      Terminer ("Incapable de crer un objet d'appel", hr);

   // Obtention de l'interface AsyncIPremier de l'objet d'appel
   // (modle VB: Finish_ ... retournera son paramtre retval
   // plutt qu'un HRESULT)
   COMAnalyserPremiers::AsyncIPremier *pAsyncPremier;
   hr = pAsyncUnk-> QueryInterface (__uuidof (COMAnalyserPremiers::AsyncIPremier),
                                    reinterpret_cast<void **>(&pAsyncPremier));
   pAsyncUnk ->Release ();
   if (FAILED(hr))
      Terminer ("Incapable d'obtenir un AsyncIPremier", hr);

   // Utilisation de l'objet d'appel asynchrone
   IdentifierDesPremiers (pAsyncPremier);
   TrouverDesPremiers (pAsyncPremier);

   pAsyncPremier-> Release ();
   CoUninitialize ();
} // main ()


// Petite procdure interactive pour identifier des entiers
// choisis par l'usager comme tant premiers ou non. Est nave
// pour ce qui est du caractre asynchrone
void IdentifierDesPremiers (COMAnalyserPremiers::AsyncIPremier *pAsyncPremier)
{
   // utilisation de l'interface IPremier pour savoir si un nombre est premier
   std::cout << "Nombre a evaluer (0 pour quitter): ";
   int NbAEvaluer;
   std::cin >> NbAEvaluer;
   std::cout << std::endl;
   while (NbAEvaluer)
   {
      unsigned char EstPremier;
      pAsyncPremier->Begin_EstPremier (NbAEvaluer);
      // synchroniser ... 
      EstPremier = pAsyncPremier->Finish_EstPremier ();
      if (EstPremier)
         std::cout << "Le nombre " << NbAEvaluer << " est premier!" << std::endl;
      else
         std::cout << "Le nombre " << NbAEvaluer << " n'est pas premier!" << std::endl;
      std::cout << "Nombre a evaluer (0 pour quitter): ";
      std::cin >> NbAEvaluer;
      std::cout << std::endl;
   }
}

// Petite procdure interactive pour trouver des entiers premiers
// (un meilleur exemple de gestion asynchrone ct client)
void TrouverDesPremiers (COMAnalyserPremiers::AsyncIPremier *pAsyncPremier)
{
   // utilisation de l'interface IPremier pour conatre le nime nombre premier
   std::cout << "Trouver le nime nombre premier... n? (0 pour quitter): ";
   int Lequel;
   std::cin >> Lequel;
   std::cout << std::endl;
   while (Lequel)
   {
      int Nombre;
      pAsyncPremier->Begin_NiemePremier (Lequel);

      // synchroniser ... Exemple banal mais concret: on affiche
      // une dizaine de points par seconde pour indiquer que le
      // traitement de la requte est en cours...
      {
         ISynchronize *pSync;
         HRESULT hr= pAsyncPremier->QueryInterface
            (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
         if (SUCCEEDED (hr))
         {
            hr = pSync->Wait (0, 0);
            while (hr != S_OK)
            {
               std::cout << '.';
               Sleep (100);
               hr = pSync->Wait (0, 0);
            }
            std::cout << std::endl;
            pSync->Release ();
         }
         else
         {
            std::cout << "Incapable d'obtenir un ISynchronize*" << std::endl;;
         }
      }
      Nombre = pAsyncPremier->Finish_NiemePremier ();
      if (Nombre != -1)
         std::cout << "Le nombre " << Nombre << " est le "
                   << Lequel << "-ime premier" << std::endl;
      else
         std::cout << "Incapable de trouver le "
                    << Lequel
                    << "-ime premier (hors bornes pour un int)"
                    << std::endl;
      std::cout << "Trouver le nime nombre premier... n? (0 pour quitter): ";
      std::cin >> Lequel;
      std::cout << std::endl;
   }
}

