#include "COMAnalystePremier.h"
#include "../../StdOutilsCOM_Lib/StdOutilsClient.h"
#include "../../ThreadManager/SystemUtilities.h"

const int COMAnalystePremier::TACHE_EST_PREMIER = 1,
          COMAnalystePremier::TACHE_NIEME_PREMIER = 2;


HRESULT COMAnalystePremier ::QueryInterface (REFIID riid, void **ppv)
{
   if (riid == IID_IUnknown)
   {
      *ppv = static_cast<IUnknown*> (static_cast<IPremier*>(this));
   }
   else if (riid == IID_IPremier)
   {
      *ppv = static_cast<IPremier*> (this);
   }
   else if (riid == IID_AsyncIPremier)
   {
      *ppv = static_cast<AsyncIPremier*> (this);
   }
   else if (riid == IID_ICallFactory)
   {
      *ppv = static_cast<ICallFactory*> (this);
   }
   else if (riid == IID_ISynchronize)
   {
      *ppv = static_cast<ISynchronize*> (this);
   }
   else if (riid == IID_ICancelMethodCalls)
   {
      *ppv = static_cast<ICancelMethodCalls*> (this);
   }
   else
   {
      // traitement d'erreur
      *ppv = 0; // NULL
      return E_NOINTERFACE;
   }
   AddRef ();
   return S_OK;
}

// -----------------------------------------------------------------
// destructeur
// -----------------------------------------------------------------
COMAnalystePremier::~COMAnalystePremier ()
{
   MortAuTravailleur (); // Dit au thread de mourir
   while (!TravailleurEstMort ()) Dormir (0); // Attend qu'il dcde
}


// ----------------------------------------------------
// Implmentation des mthodes de la classe COMAnalystePremier 
// (pour AsyncIPremier) (version nave)
// ----------------------------------------------------

HRESULT __stdcall COMAnalystePremier ::Begin_EstPremier( int Nb )
{
   // viter plusieurs appels concurrents sur le mme objet d'appel
   if (m_PretPourTraitement)
      return E_FAIL;

   // Remettre  jour la synchro
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   if (FAILED (hr))
      return E_FAIL;
   pSync->Reset ();
   pSync->Release ();

   // Saisie des intrants
   m_TacheEnCours = TACHE_EST_PREMIER;
   m_Intrant = Nb;
   // idalement, faudrait un mutex pour viter que deux appels soient
   // soumis  cet objet d'appel entre le dbut de la mthode et cette
   // ligne-ci...
   m_PretPourTraitement = true;

   return S_OK;
}

HRESULT __stdcall COMAnalystePremier ::Finish_EstPremier( boolean *retval)
{
   // Remettre  jour la synchro
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   if (FAILED (hr))
      return E_FAIL;
   pSync->Wait (0, INFINITE);
   pSync->Release ();

   // Offrir les extrants, et permettre un nouvel appel
   *retval = (m_Extrant? TRUE : FALSE);
   m_PretPourTraitement = false;
   return S_OK;
}

HRESULT __stdcall COMAnalystePremier::Begin_NiemePremier( int Lequel )
{
   // viter plusieurs appels concurrents sur le mme objet d'appel
   if (m_PretPourTraitement)
      return E_FAIL;

   // Remettre  jour la synchro
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   if (FAILED (hr))
      return E_FAIL;
   pSync->Reset ();
   pSync->Release ();

   // Saisie des intrants
   m_TacheEnCours = TACHE_NIEME_PREMIER;
   m_Intrant = Lequel;
   // idalement, faudrait un mutex pour viter que deux appels soient
   // soumis  cet objet d'appel entre le dbut de la mthode et cette
   // ligne-ci...
   m_PretPourTraitement = true;

   return S_OK;
}

HRESULT __stdcall COMAnalystePremier::Finish_NiemePremier( int *retval)
{
   // Remettre  jour la synchro
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   if (FAILED (hr))
      return E_FAIL;
   pSync->Wait (0, INFINITE);
   pSync->Release ();

   // Offrir les extrants, et permettre un nouvel appel
   *retval = m_Extrant;
   m_PretPourTraitement = false;
   return S_OK;
}

// ----------------------------------------------------
// Implmentation des mthodes de la classe COMAnalystePremier 
// (pour ICallFactory) (version nave)
// ----------------------------------------------------

unsigned long __stdcall Travailleur (void *p)
{
   // Charger COM pour ce thread. Le destructeur de ChargeurCOM
   // dcharge COM automatiquement (vieux truc scout).
   ChargeurCOM cc; 

   // Le thread reoit mon serveur en paramtre. On pourrait
   // faire mieux en lui laissant accder  une file d'attente
   // globale d'objets d'appel...
   COMAnalystePremier *srv = reinterpret_cast<COMAnalystePremier *>(p);
   while (!srv->TravailleurDoitMourir ())
   {
      if (srv->PretPourTraitement ())
      {
         switch (srv->GetTacheEnCours ())
         {
         case COMAnalystePremier::TACHE_EST_PREMIER:
            srv->Faire_EstPremier ();
            break;
         case COMAnalystePremier::TACHE_NIEME_PREMIER:
            srv->Faire_NiemePremier ();
            break;
         default: //  voir
            ;
         }

         srv->TraitementTermine ();
      }
      Dormir (1);
   }
   // Lorsque le thread s'apprte  mourir, il signale sa mort.
   // Ceci est utilis par le destructeur du serveur comme un
   // signal qu'il peut lui-mme terminer sa destruction.
   //
   // Faut pas que le serveur ne meurt avant que le thread n'ait
   // indiqu qu'il n'a plus recours  ses services!
   srv->TravailleurDecede ();
   return 0L;
}


// -----------------------------------------------------------------
// Implantation du code de EstPremier()  proprement dit. Utilise
// les attributs de contrle m_Intrant et m_Extrant, qui reprsentent
// les paramtres [in] et [out] de l'appel.
//
// On pourrait faire beaucoup mieux, mais a marche dans notre petit
// contexte simplifi.
// -----------------------------------------------------------------
void COMAnalystePremier ::Faire_EstPremier ()
{
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   boolean Resultat;
   EstPremier (m_Intrant, &Resultat);
   m_Extrant = Resultat;
   pSync->Signal ();
   pSync->Release ();
}

// -----------------------------------------------------------------
// Implantation du code de NiemePremier()  proprement dit. Utilise
// les attributs de contrle m_Intrant et m_Extrant, qui reprsentent
// les paramtres [in] et [out] de l'appel.
//
// On pourrait faire beaucoup mieux, mais a marche dans notre petit
// contexte simplifi.
// -----------------------------------------------------------------
void COMAnalystePremier ::Faire_NiemePremier ()
{
   ISynchronize *pSync;
   HRESULT hr = static_cast<AsyncIPremier *> (this) -> QueryInterface
                  (IID_ISynchronize, reinterpret_cast<void **> (&pSync));
   NiemePremier (m_Intrant, &m_Extrant);
   pSync->Signal ();
   pSync->Release ();
}




HRESULT __stdcall COMAnalystePremier ::CreateCall (REFIID riid,
                                                   IUnknown *pCtrlUnk,
                                                   REFIID riid2,
                                                   IUnknown **ppv)
{
   HRESULT hr;

   // Surveiller le support  l'agrgation (voir serveur excutable).
   if (pCtrlUnk)
   {
      *ppv = 0;
      hr = CLASS_E_NOAGGREGATION;
   }
   // On ne supporte ici qu'une seule interface asynchrone; si on
   // en supportait plusieurs, faudrait les passer en revue une  une...
   else if (riid == IID_AsyncIPremier)
   {
      hr = QueryInterface (riid2, reinterpret_cast<void **> (ppv));
      // On cre un thread travailleur pou grer l'objet d'appel, du
      // moins si ce thread n'avait pas encore t lanc...
      if (SUCCEEDED (hr) && !m_Travailleur)
      {
         DWORD Id; // pas utilis, sauf pour rsoudre l'appel ci-dessous
         m_Travailleur = CreateThread (0, 0, Travailleur, this, 0, &Id);
      }
   }
   else // interface non supporte.., a peut arriver!
   {
      *ppv = 0;
      hr = E_NOINTERFACE;
   }
   return hr;
}

// pour ICancelMethodCalls (j'ai t paresseux, mais faudrait
// vraiment l'implanter).
HRESULT __stdcall COMAnalystePremier ::Cancel(ULONG ulSeconds)
{
   return S_OK;
}

// pour ICancelMethodCalls (j'ai t paresseux, mais faudrait
// vraiment l'implanter).
HRESULT __stdcall COMAnalystePremier ::TestCancel()
{
   return S_FALSE;
}


// Pour ISynchronize. Essentiel.
HRESULT __stdcall COMAnalystePremier ::Wait( DWORD dwFlags, DWORD dwMilliseconds)
{
   // Si le dlai est INFINITE, on boucle et on fait dodo le temps
   // que le traitement de la tche en cours soit complt
   if (dwMilliseconds == INFINITE)
   {
      while (!m_Termine) Dormir (0);
   }
   // Sinon, on boucle et on fait dodo le temps demand ou jusqu'
   // ce que le traitement de la tche en cours soit complt
   else
   {
      while (dwMilliseconds && !m_Termine)
      {
         dwMilliseconds --;
         Dormir (1);
      }
   }
   return m_Termine ? S_OK : S_FALSE;
}

// Appel par le serveur pour signaler la compltion d'un traitement
// de tche asynchrone
HRESULT __stdcall COMAnalystePremier ::Signal()
{
   m_Termine = true;
   return S_OK;
}

// Appel par le serveur pour signaler le dbut d'un traitement de
// tche asynchrone
HRESULT __stdcall COMAnalystePremier ::Reset()
{
   m_Termine = false;
   return S_OK;
}
