#ifndef COMANALYSTEPREMIER_H
#define COMANALYSTEPREMIER_H

#include "IPremierImpl.h"
#include "../Commun/premier.h"
#include "../../StdOutilsCOM_Lib/IUnknownImpl.h"

// ----------------------------------------------------
// ----------------------------------------------------
// ----------------------------------------------------
// Class COMAnalystePremier : implmentation encapsule
//                            de IPremier
//
// Pour le serveur interne, j'ai implant une version
// plus ou moins complte (j'ai omis une vraie
// implantation de ICancelMethodCalls, en particulier).
//
// Ici, un seul objet joue le rle d'objet d'appel et
// de serveur  proprement dit. Ceci n'est pas une
// bonne ide dans la vraie vie, mais a marche si on
// n'a qu'un seul appel  la fois  traiter.
// ----------------------------------------------------

class COMAnalystePremier : virtual public IUnknownImpl,
                           virtual public IPremierImpl,
                           virtual public AsyncIPremier,
                           virtual public ICallFactory,
                           virtual public ISynchronize,
                           virtual public ICancelMethodCalls
{
public:
   // Pour IUnknown
   unsigned long __stdcall AddRef () { return IUnknownImpl::AddRef (); }
   unsigned long __stdcall Release () { return IUnknownImpl::Release (); }
   HRESULT __stdcall QueryInterface (REFIID riid, void **ppv);

   // Pour AsyncIPremier
   HRESULT __stdcall Begin_EstPremier( int Nb );
   HRESULT __stdcall Finish_EstPremier( boolean *retval);
   HRESULT __stdcall Begin_NiemePremier( int Lequel );
   HRESULT __stdcall Finish_NiemePremier( int *retval);
   
   // Pour ICallFactory
   HRESULT __stdcall CreateCall (REFIID riid,
                                 IUnknown *pCtrlUnk,
                                 REFIID riid2,
                                 IUnknown **ppv);
   // Pour ISynchronize
   
   HRESULT __stdcall Wait( DWORD dwFlags, DWORD dwMilliseconds);
   HRESULT __stdcall Signal();

   HRESULT __stdcall Reset();


   // pour ICancelMethodCalls
   HRESULT __stdcall Cancel(ULONG ulSeconds);
   HRESULT __stdcall TestCancel();

   // -----------------------------------------------------------------
   // constructeur
   // -----------------------------------------------------------------
   // Les fanions boolens initialiss ici servent  la gestion du
   // thread Travailleur... Le HANDLE du thread est initialis  0
   COMAnalystePremier () : IUnknownImpl (), m_PretPourTraitement (false),
                                            m_MortAuTravailleur  (false),
                                            m_TravailleurDecede  (false),
                                            m_Termine (false),
                                            m_Travailleur (0)
   {
   };

   // -----------------------------------------------------------------
   // destructeur
   // -----------------------------------------------------------------
   ~COMAnalystePremier ();

   // -----------------------------------------------------------------
   // Support  la gestion du thread Travailleur
   // -----------------------------------------------------------------
   static const int
      TACHE_EST_PREMIER,
      TACHE_NIEME_PREMIER;
   int GetTacheEnCours () const { return m_TacheEnCours; }
   void MortAuTravailleur () { m_MortAuTravailleur = true; }
   bool TravailleurDoitMourir () const { return m_MortAuTravailleur; }
   void TravailleurDecede () { m_TravailleurDecede = true; }
   bool TravailleurEstMort () const { return m_TravailleurDecede; }
   bool PretPourTraitement () const { return m_PretPourTraitement; }
   void TraitementTermine () { m_PretPourTraitement = false; }

   // -----------------------------------------------------------------
   // Implantation du code de EstPremier()  proprement dit. Utilise
   // les attributs de contrle m_Intrant et m_Extrant, qui reprsentent
   // les paramtres [in] et [out] de l'appel.
   //
   // On pourrait faire beaucoup mieux, mais a marche dans notre petit
   // contexte simplifi.
   // -----------------------------------------------------------------
   void Faire_EstPremier ();

   // -----------------------------------------------------------------
   // Implantation du code de NiemePremier()  proprement dit. Utilise
   // les attributs de contrle m_Intrant et m_Extrant, qui reprsentent
   // les paramtres [in] et [out] de l'appel.
   //
   // On pourrait faire beaucoup mieux, mais a marche dans notre petit
   // contexte simplifi.
   // -----------------------------------------------------------------
   void Faire_NiemePremier ();

private:
   int m_Intrant; // intrant de la demande de service en traitement
   int m_Extrant; // extrant de la demande de service la plus rcemment traite
   bool m_PretPourTraitement; // un appel a t soumis
   int m_TacheEnCours; // la tche en cours de traitement
   HANDLE m_Travailleur; // le thread Travailleur et ses fanions
   bool m_MortAuTravailleur,
        m_TravailleurDecede,
        m_Termine;
};

#endif // !defined COMANALYSTEPREMIER_H
